#include "project.h"

#define DELTA_VALUE   10
#define PWM_PERIOD  1000
#define MODE_NORMAL    0
#define MODE_TWICE     1
#define MODE_HALF      2

#define SW_OFF         1
#define SW_ON          0

#define LED_OFF        0
#define LED_ON         1

int count =           0 ;
int period = PWM_PERIOD ;
int comp =   PWM_PERIOD ;
int delta =  DELTA_VALUE ;
int mode =   MODE_NORMAL ;

void advance_state_machine(void)
{
    switch(mode) {
    case MODE_NORMAL:
        mode = MODE_TWICE ;
        delta = 2 * DELTA_VALUE ;
        break ;
    case MODE_TWICE:
        mode = MODE_HALF ;
        delta = DELTA_VALUE / 2 ;
        break ;
    case MODE_HALF:
    default:
        mode = MODE_NORMAL ;
        delta = DELTA_VALUE ;
        break ;
    }    
}

void cruise(void)
{
    int phase ;
    
    phase = count % period ;
    
    if (phase < comp) {
        LED_Write(LED_ON) ;
    } else {
        LED_Write(LED_OFF) ;
    }
    
    if (phase == 0) {
        comp -= delta ;
        if (comp <= 0) {
            comp = period ;
        }
    } 
    count = (count + 1) % (PWM_PERIOD * DELTA_VALUE) ;
}

int main(void)
{
    int prev_sw ;
    int sw ;

    CyGlobalIntEnable; /* Enable global interrupts. */
        
    prev_sw = SW1_Read() ;

    for(;;) {
        sw = SW1_Read() ;
        
        if (sw == SW_ON) { /* when switch is pressed turn LED OFF */
            LED_Write(LED_OFF) ;
        } else if ((sw != prev_sw)&&(sw == SW_OFF)) { /* SW is just released */
            advance_state_machine() ;
        } else {
            cruise() ;
        }
        
        prev_sw = sw ;
        CyDelayUs(1) ;
    }
}
